<?php

namespace SimpleDBAL;


class QueryBuilder
{

    private $db;

    private $select = [];

    private $table = null;

    private $values = [];

    private $where = [];

    private $orderBy = [];

    private $join = [];

    private $paramters = [];

    private $groupBy = null;

    private $qType;

    public const IS_SELECT = 1;
    public const IS_INSERT = 2;
    public const IS_UPDATE = 3;
    public const IS_DELETE = 4;
    public const IS_TRUNCATE = 5;

    public function __construct(Database $db)
    {
        $this->db = $db;
    }

    public function Execute($fetchArray = false)
    {
        echo var_dump(SQLBuilder::Build($this));
        $query = $this->db->prepare(SQLBuilder::Build($this));
        if ($this->paramters) {
            foreach ($this->paramters as $paraKey => $paraValue) {
                $query->bindValue($paraKey, $paraValue);
            }
        }
        $query->execute();
            if ($query->rowCount() > 1 || $fetchArray == true) {
                return $query->fetchAll($this->db::FETCH_ASSOC);
            } else {
                return $query->fetch($this->db::FETCH_ASSOC);
            }
    }

    public function Reset()
    {
        $this->select = [];
        $this->table = null;
        $this->values = [];
        $this->where = [];
        $this->orderBy = [];
        $this->join = [];
        $this->paramters = [];
        $this->groupBy = null;
    }

    /**
     * Nötig für ein SELECT Query. Speichert in einem Array die benötigten Felder des SQL Statements
     * @param mixed ...$select
     * @return QueryBuilder
     */
    public function Select(...$select): QueryBuilder
    {
        foreach ($select as $item)
            $this->select[] = $item;
        $this->qType = self::IS_SELECT;
        return $this;
    }

    /**
     * Nötig für ein UPDATE Query. Speichert in einem Array die benötigten Felder des SQL Statements
     * @param array $update
     * @return QueryBuilder
     */
    public function Update(array $update): QueryBuilder
    {
        $this->select = $update;
        $this->qType = self::IS_UPDATE;
        return $this;
    }

    /**
     * Nötig für einen INSERT Query. Speichert in einem Array die nötigen felder des SQL Statements
     * @param array $insert
     * @return QueryBuilder
     */
    public function Insert(array $insert): QueryBuilder
    {
        $this->select = $insert;
        $this->qType = self::IS_INSERT;
        return $this;
    }

    /**
     * Nötig für ein DELETE Query.
     * @return QueryBuilder
     */
    public function Delete(): QueryBuilder
    {
        $this->qType = self::IS_DELETE;
        return $this;
    }

    public function Truncate(): QueryBuilder
    {
        $this->qType = self::IS_TRUNCATE;
        return $this;
    }

    /**
     * Nötig um die Tabelle der Datenbank auszuwählen die benötigt wird
     * @param string $table
     * @return QueryBuilder
     */
    public function Table(string $table): QueryBuilder
    {
        $this->table = $table;
        return $this;
    }

    public function InsertValues(array $insertValues): QueryBuilder
    {
        $this->values = $insertValues;
        return $this;
    }

    /**
     * Nötig wenn man auf eine andere Tabelle Joinen möchte. Speichert
     * Die nötigen Angaben in einem Array welches vom SQLBuilder verarbeitet wird.
     * @param $joinType
     * @param mixed ...$jArgs Following order: joinTable, joinColumn, originTable, originColumn
     * @return QueryBuilder
     */
    public function Join($joinType, ...$jArgs): QueryBuilder
    {
        $this->join[$joinType] = $jArgs;
        return $this;
    }

    /**
     * Nötig für SET Angaben im UPDATE Segment eines SQL Statements.
     * Nutzt das Select-Array, da während eines UPDATE keine SELECT Angaben getätigt werden
     * @param $column
     * @param $value
     * @return QueryBuilder
     */
    public function Set(string $column, $value): QueryBuilder
    {
        $this->select[$column] = $value;
        return $this;
    }

    /**
     * Nötig für WHERE Teil des SQL Statements. Speichert die Angaben in dem Array $this->where für weitere Verarbeitung
     * @param $column
     * @param $value
     * @param string $operator
     * @return QueryBuilder
     */
    public function Where($column, $value, $operator = "="): QueryBuilder
    {
        $this->where[] = (new QueryFilter($column, $value))->setOperator($operator);
        return $this;
    }

    /**
     * Angabe nach welchen Spalten sortiert werden soll
     * @param $order
     * @param bool $desc
     * @return QueryBuilder
     */
    public function OrderBy($order, bool $desc = false): QueryBuilder
    {
        if (is_array($order)) {
            foreach ($order as $column => $desc) {
                if (is_bool($desc))
                    $this->orderBy[$column] = $desc;
                else
                    $this->orderBy[$desc] = false;
            }
        } else {
            $this->orderBy[$order] = $desc;
        }
        return $this;
    }

    public function GroupBy($value)
    {
        $this->groupBy = $value;
        return $this;
    }

    public function setParameter($param, $value)
    {
        $this->paramters[$param] = $value;
        return $this;
    }

    /**
     * @return array
     */
    public function getSelect(): array
    {
        return $this->select;
    }

    /**
     * @return null
     */
    public function getTable()
    {
        return $this->table;
    }

    /**
     * @return array
     */
    public function getValues(): array
    {
        return $this->values;
    }

    /**
     * @return array
     */
    public function getWhere(): array
    {
        return $this->where;
    }

    /**
     * @return array
     */
    public function getJoin(): array
    {
        return $this->join;
    }

    /**
     * @return array
     */
    public function getOrderBy(): array
    {
        return $this->orderBy;
    }

    /**
     * @return string
     */
    public function getGroupBy()
    {
        return $this->groupBy;
    }

    /**
     * @return mixed
     */
    public function getQType()
    {
        return $this->qType;
    }

}